@@ -1,12 +1,11 @@
 /* random.c  -	random number generator
- * Copyright (C) 1998, 1999, 2000, 2001, 2002,
- *               2003, 2006 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
+ * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -15,7 +14,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
 
@@ -33,27 +33,22 @@
 #include <errno.h>
 #include <string.h>
 #include <time.h>
-#ifndef _WIN32
 #include <sys/time.h>
-#endif
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <fcntl.h>
-#ifdef HAVE_GETHRTIME
-#include <sys/times.h>
+#ifdef	HAVE_GETHRTIME
+  #include <sys/times.h>
 #endif
 #ifdef HAVE_GETTIMEOFDAY
-#include <sys/time.h>
-#endif
-#ifdef HAVE_TIMES
-#include <sys/times.h>
+  #include <sys/times.h>
 #endif
 #ifdef HAVE_GETRUSAGE
-#include <sys/resource.h>
+  #include <sys/resource.h>
 #endif
-#ifdef _WIN32
-#include <process.h>
+#ifdef __MINGW32__
+  #include <process.h>
 #endif
 #include "util.h"
 #include "rmd.h"
@@ -63,35 +58,17 @@
 #include "rand-internal.h"
 #include "algorithms.h"
 
-#ifdef __VMS
-# include <rmsdef.h>
-# include "vms.h"
-#endif /* def __VMS */
-
 #ifndef RAND_MAX   /* for SunOS */
-#define RAND_MAX 32767
-#endif
-
-/* 2008-03-31  SMS.
- * VMS C RTL before V8.3 lacks byte-range file locking, but by default,
- * a file opened for write access is not shared, so mutual exclusion can
- * most generally be handled at the open().  */
-
-/* Check whether we can lock the seed file read write. */
-#if defined(HAVE_FCNTL) && defined(HAVE_FTRUNCATE)      \
-  && !defined(HAVE_W32_SYSTEM) && !defined(__VMS)
-#define LOCK_SEED_FILE 1
-#else
-#define LOCK_SEED_FILE 0
+  #define RAND_MAX 32767
 #endif
 
 
 #if SIZEOF_UNSIGNED_LONG == 8
-#define ADD_VALUE 0xa5a5a5a5a5a5a5a5
+  #define ADD_VALUE 0xa5a5a5a5a5a5a5a5
 #elif SIZEOF_UNSIGNED_LONG == 4
-#define ADD_VALUE 0xa5a5a5a5
+  #define ADD_VALUE 0xa5a5a5a5
 #else
-#error weird size for an unsigned long
+  #error weird size for an unsigned long
 #endif
 
 #define BLOCKLEN  64   /* hash this amount of bytes */
@@ -104,7 +81,7 @@
 #define POOLBLOCKS 30
 #define POOLSIZE (POOLBLOCKS*DIGESTLEN)
 #if (POOLSIZE % SIZEOF_UNSIGNED_LONG)
-#error Please make sure that poolsize is a multiple of ulong
+  #error Please make sure that poolsize is a multiple of ulong
 #endif
 #define POOLWORDS (POOLSIZE / SIZEOF_UNSIGNED_LONG)
 
@@ -121,7 +98,6 @@ static int just_mixed;
 static int did_initial_extra_seeding;
 static char *seed_file_name;
 static int allow_seed_file_update;
-static int no_seed_file_locking;
 
 static int secure_alloc;
 static int quick_test;
@@ -155,12 +131,12 @@ getfnc_gather_random (void))(void (*)(const void*, size_t, int), int,
 {
 #ifdef USE_ALL_RANDOM_MODULES
   static int (*fnc)(void (*)(const void*, size_t, int), int, size_t, int);
-
+  
   if (fnc)
     return fnc;
 # ifdef USE_RNDLINUX
   if ( !access (NAME_OF_DEV_RANDOM, R_OK)
-       && !access (NAME_OF_DEV_URANDOM, R_OK))
+       && !access (NAME_OF_DEV_RANDOM, R_OK))
     {
       fnc = rndlinux_gather_random;
       return fnc;
@@ -187,9 +163,6 @@ getfnc_gather_random (void))(void (*)(const void*, size_t, int), int,
 # ifdef USE_RNDUNIX
   return rndunix_gather_random;
 # endif
-# ifdef USE_RNDVMS
-  return rndvms_gather_random;
-# endif
 # ifdef USE_RNDEGD
   return rndegd_gather_random;
 # endif
@@ -203,7 +176,7 @@ getfnc_gather_random (void))(void (*)(const void*, size_t, int), int,
   return NULL;
 }
 
-static int (*
+static void (*
 getfnc_fast_random_poll (void))( void (*)(const void*, size_t, int), int)
 {
 #ifdef USE_RNDW32
@@ -220,10 +193,10 @@ initialize(void)
     /* The data buffer is allocated somewhat larger, so that
      * we can use this extra space (which is allocated in secure memory)
      * as a temporary hash buffer */
-    rndpool = secure_alloc ? xmalloc_secure_clear(POOLSIZE+BLOCKLEN)
-			   : xmalloc_clear(POOLSIZE+BLOCKLEN);
-    keypool = secure_alloc ? xmalloc_secure_clear(POOLSIZE+BLOCKLEN)
-			   : xmalloc_clear(POOLSIZE+BLOCKLEN);
+    rndpool = secure_alloc ? m_alloc_secure_clear(POOLSIZE+BLOCKLEN)
+			   : m_alloc_clear(POOLSIZE+BLOCKLEN);
+    keypool = secure_alloc ? m_alloc_secure_clear(POOLSIZE+BLOCKLEN)
+			   : m_alloc_clear(POOLSIZE+BLOCKLEN);
     is_initialized = 1;
 }
 
@@ -231,8 +204,8 @@ static void
 burn_stack (int bytes)
 {
     char buf[128];
-
-    wipememory(buf,sizeof buf);
+    
+    memset (buf, 0, sizeof buf);
     bytes -= sizeof buf;
     if (bytes > 0)
         burn_stack (bytes);
@@ -251,7 +224,7 @@ random_dump_stats()
 }
 
 void
-secure_randoxmalloc()
+secure_random_alloc()
 {
     secure_alloc = 1;
 }
@@ -280,19 +253,7 @@ randomize_buffer( byte *buffer, size_t length, int level )
 {
     char *p = get_random_bits( length*8, level, 1 );
     memcpy( buffer, p, length );
-    xfree(p);
-}
-
-
-/* Randomize the MPI by setting it to NBITS of random of quality LEVEL.  */
-void
-randomize_mpi (MPI mpi, size_t nbits, int level)
-{
-  unsigned char *buffer;
-
-  buffer = get_random_bits (nbits, level, mpi_is_secure (mpi));
-  mpi_set_buffer (mpi, buffer, (nbits+7)/8, 0);
-  xfree (buffer);
+    m_free(p);
 }
 
 
@@ -304,13 +265,6 @@ random_is_faked()
     return faked_rng || quick_test;
 }
 
-/* Disable locking of seed files. */
-void
-random_disable_locking ()
-{
-  no_seed_file_locking = 1;
-}
-
 /****************
  * Return a pointer to a randomized buffer of level 0 and LENGTH bits
  * caller must free the buffer.
@@ -334,7 +288,7 @@ get_random_bits( size_t nbits, int level, int secure )
 	rndstats.ngetbytes2++;
     }
 
-    buf = secure && secure_alloc ? xmalloc_secure( nbytes ) : xmalloc( nbytes );
+    buf = secure && secure_alloc ? m_alloc_secure( nbytes ) : m_alloc( nbytes );
     for( p = buf; nbytes > 0; ) {
 	size_t n = nbytes > POOLSIZE? POOLSIZE : nbytes;
 	read_pool( p, n, level );
@@ -357,24 +311,26 @@ mix_pool(byte *pool)
     RMD160_CONTEXT md;
 
     rmd160_init( &md );
-#if DIGESTLEN != 20
-#error must have a digest length of 20 for ripe-md-160
-#endif
-    /* pool -> pool' */
+ #if DIGESTLEN != 20
+    #error must have a digest length of 20 for ripe-md-160
+ #endif
+    /* loop over the pool */
     pend = pool + POOLSIZE;
     memcpy(hashbuf, pend - DIGESTLEN, DIGESTLEN );
     memcpy(hashbuf+DIGESTLEN, pool, BLOCKLEN-DIGESTLEN);
     rmd160_mixblock( &md, hashbuf);
-    memcpy(pool, hashbuf, DIGESTLEN);
+    memcpy(pool, hashbuf, 20 );
 
-    /* Loop for the remaining iterations.  */
     p = pool;
     for( n=1; n < POOLBLOCKS; n++ ) {
-	if( p + BLOCKLEN < pend )
-	    memcpy(hashbuf, p, BLOCKLEN);
+	memcpy(hashbuf, p, DIGESTLEN );
+
+	p += DIGESTLEN;
+	if( p+DIGESTLEN+BLOCKLEN < pend )
+	    memcpy(hashbuf+DIGESTLEN, p+DIGESTLEN, BLOCKLEN-DIGESTLEN);
 	else {
-	    char *pp = p;
-	    for(i=0; i < BLOCKLEN; i++ ) {
+	    char *pp = p+DIGESTLEN;
+	    for(i=DIGESTLEN; i < BLOCKLEN; i++ ) {
 		if( pp >= pend )
 		    pp = pool;
 		hashbuf[i] = *pp++;
@@ -382,8 +338,7 @@ mix_pool(byte *pool)
 	}
 
 	rmd160_mixblock( &md, hashbuf);
-        p += DIGESTLEN;
-	memcpy(p, hashbuf, DIGESTLEN);
+	memcpy(p, hashbuf, 20 );
     }
     burn_stack (384); /* for the rmd160_mixblock() */
 }
@@ -394,53 +349,9 @@ set_random_seed_file( const char *name )
 {
     if( seed_file_name )
 	BUG();
-    seed_file_name = xstrdup( name );
+    seed_file_name = m_strdup( name );
 }
 
-
-/* Lock an open file identified by file descriptor FD and wait a
-   reasonable time to succeed.  With FOR_WRITE set to true a Rite lock
-   will be taken.  FNAME is used only for diagnostics. Returns 0 on
-   success or -1 on error. */
-static int
-lock_seed_file (int fd, const char *fname, int for_write)
-{
-#if LOCK_SEED_FILE
-  struct flock lck;
-  struct timeval tv;
-  int backoff=0;
-
-  if (no_seed_file_locking)
-    return 0;
-
-  /* We take a lock on the entire file. */
-  memset (&lck, 0, sizeof lck);
-  lck.l_type = for_write? F_WRLCK : F_RDLCK;
-  lck.l_whence = SEEK_SET;
-
-  while (fcntl (fd, F_SETLK, &lck) == -1)
-    {
-      if (errno != EAGAIN && errno != EACCES)
-        {
-          log_info (_("can't lock `%s': %s\n"), fname, strerror (errno));
-          return -1;
-        }
-
-      if (backoff > 2) /* Show the first message after ~3.75 seconds. */
-        log_info( _("waiting for lock on `%s'...\n"), fname);
-
-      tv.tv_sec = backoff;
-      tv.tv_usec = 250000;
-      select (0, NULL, NULL, NULL, &tv);
-      if (backoff < 10)
-        backoff++ ;
-    }
-#endif /*LOCK_SEED_FILE*/
-  return 0;
-}
-
-
-
 /****************
  * Read in a seed form the random_seed file
  * and return true if this was successful
@@ -456,14 +367,11 @@ read_seed_file(void)
     if( !seed_file_name )
 	return 0;
 
-#if defined(HAVE_DOSISH_SYSTEM) || defined(__CYGWIN__)
+  #ifdef HAVE_DOSISH_SYSTEM
     fd = open( seed_file_name, O_RDONLY | O_BINARY );
-#elif defined( __VMS)
-    /* We're only reading, so allow others to do anything. */
-    fd = open( seed_file_name, O_RDONLY, 0777, "shr=get,put,upd" );
-#else
+  #else
     fd = open( seed_file_name, O_RDONLY );
-#endif
+  #endif
     if( fd == -1 && errno == ENOENT) {
 	allow_seed_file_update = 1;
 	return 0;
@@ -473,12 +381,6 @@ read_seed_file(void)
 	log_info(_("can't open `%s': %s\n"), seed_file_name, strerror(errno) );
 	return 0;
     }
-    if (lock_seed_file (fd, seed_file_name, 0))
-      {
-        close (fd);
-        return 0;
-      }
-
     if( fstat( fd, &sb ) ) {
 	log_info(_("can't stat `%s': %s\n"), seed_file_name, strerror(errno) );
 	close(fd);
@@ -500,38 +402,14 @@ read_seed_file(void)
 	close(fd);
 	return 0;
     }
-
     do {
 	n = read( fd, buffer, POOLSIZE );
     } while( n == -1 && errno == EINTR );
-    /* The N==0, ENOENT, and N!=POOLSIZE cases may happen if another
-       process is updating the file.  For consistency we use the same
-       recovery strategy as with the pre-read checks.  */
-    if (!n) {
-        log_info(_("note: random_seed file is empty\n") );
-        allow_seed_file_update = 1;
-        close(fd);
-        return 0;
-    }
-    else if( n == -1 && errno == ENOENT) {
-        /* On a Unix system that should never happen.  However, I can
-           imagine this error code on non-inode based systems.  */
-	log_info(_("can't read `%s': %s\n"), seed_file_name, strerror(errno));
-        allow_seed_file_update = 1;
-	close(fd);
-	return 0;
-    }
-    else if( n == -1 ) {
-        /* A real read error.  */
+    if( n != POOLSIZE ) {
 	log_fatal(_("can't read `%s': %s\n"), seed_file_name,strerror(errno) );
 	close(fd);
 	return 0;
     }
-    else if ( n != POOLSIZE ) {
-        log_info(_("WARNING: invalid size of random_seed file - not used\n") );
-	close(fd);
-	return 0;
-    }
 
     close(fd);
 
@@ -549,8 +427,8 @@ read_seed_file(void)
     /* And read a few bytes from our entropy source.  By using
      * a level of 0 this will not block and might not return anything
      * with some entropy drivers, however the rndlinux driver will use
-     * /dev/urandom and return some stuff - Do not read too much as we
-     * want to be friendly to the scarce system entropy resource. */
+     * /dev/urandom and return some stuff - Do not read to much as we
+     * want to be friendly to the scare system entropy resource. */
     read_random_source( 0, 16, 0 );
 
     allow_seed_file_update = 1;
@@ -579,64 +457,16 @@ update_random_seed_file()
     mix_pool(rndpool); rndstats.mixrnd++;
     mix_pool(keypool); rndstats.mixkey++;
 
-#if defined(HAVE_DOSISH_SYSTEM) || defined(__CYGWIN__)
+  #ifdef HAVE_DOSISH_SYSTEM
     fd = open( seed_file_name, O_WRONLY|O_CREAT|O_TRUNC|O_BINARY,
 							S_IRUSR|S_IWUSR );
-#else
-# if LOCK_SEED_FILE
-    fd = open( seed_file_name, O_WRONLY|O_CREAT, S_IRUSR|S_IWUSR );
-# else
-#  ifdef __VMS
-    /* Open the seed file for exclusive write access, but allow other
-     * readers.  Loop until success.  Complain after a few failures.  */
-    {
-        int backoff = 0;
-
-        while ((fd = open( seed_file_name,
-                           O_WRONLY|O_CREAT,
-                           S_IRUSR|S_IWUSR,
-                           "shr=get")) == -1 )
-        {
-          if ((errno != EVMSERR) || (vaxc$errno != RMS$_FLK))
-            {
-              /* Some unexpected open failure. */
-              log_info (_("can't lock `%s': %s\n"),
-                        seed_file_name, strerror (errno));
-              return;
-            }
-
-          if (backoff > 2) /* Show the first message after ~3.75 seconds. */
-            log_info( _("waiting for lock on `%s'...\n"), seed_file_name);
-
-          wait_vms( backoff+ 0.25);
-          if (backoff < 10)
-            backoff++ ;
-        }
-    }
-#  else /* !def __VMS */
+  #else
     fd = open( seed_file_name, O_WRONLY|O_CREAT|O_TRUNC, S_IRUSR|S_IWUSR );
-#  endif /* !def __VMS */
-# endif
-#endif
+  #endif
     if( fd == -1 ) {
-	log_info(_("can't create `%s': %s\n"), seed_file_name, strerror(errno));
+	log_info(_("can't create `%s': %s\n"), seed_file_name, strerror(errno) );
 	return;
     }
-
-    if (lock_seed_file (fd, seed_file_name, 1))
-      {
-        close (fd);
-        return;
-      }
-#if LOCK_SEED_FILE
-    if (ftruncate (fd, 0))
-      {
-	log_info(_("can't write `%s': %s\n"), seed_file_name, strerror(errno));
-        close (fd);
-        return;
-      }
-#endif /*LOCK_SEED_FILE*/
-
     do {
 	i = write( fd, keypool, POOLSIZE );
     } while( i == -1 && errno == EINTR );
@@ -734,7 +564,7 @@ read_pool( byte *buffer, size_t length, int level )
 	if( pool_balance < 0 )
 	    pool_balance = 0;
 	/* and clear the keypool */
-	wipememory(keypool, POOLSIZE);
+	memset( keypool, 0, POOLSIZE );
     }
 }
 
@@ -783,7 +613,7 @@ random_poll()
 void
 fast_random_poll()
 {
-    static int (*fnc)( void (*)(const void*, size_t, int), int) = NULL;
+    static void (*fnc)( void (*)(const void*, size_t, int), int) = NULL;
     static int initialized = 0;
 
     rndstats.fastpolls++;
@@ -799,53 +629,54 @@ fast_random_poll()
     }
 
     /* fall back to the generic function */
-#if defined(HAVE_GETHRTIME) && !defined(HAVE_BROKEN_GETHRTIME)
+  #if defined(HAVE_GETHRTIME) && !defined(HAVE_BROKEN_GETHRTIME)
     {	hrtime_t tv;
-        /* On some Solaris and HPUX system gethrtime raises an SIGILL, but we
+        /* On some Solaris and HPUX system gethrtime raises an SIGILL, but we 
          * checked this with configure */
 	tv = gethrtime();
 	add_randomness( &tv, sizeof(tv), 1 );
     }
-#elif defined (HAVE_GETTIMEOFDAY)
+  #elif defined (HAVE_GETTIMEOFDAY)
     {	struct timeval tv;
 	if( gettimeofday( &tv, NULL ) )
 	    BUG();
 	add_randomness( &tv.tv_sec, sizeof(tv.tv_sec), 1 );
 	add_randomness( &tv.tv_usec, sizeof(tv.tv_usec), 1 );
     }
-#elif defined (HAVE_CLOCK_GETTIME)
+  #elif defined (HAVE_CLOCK_GETTIME)
     {	struct timespec tv;
 	if( clock_gettime( CLOCK_REALTIME, &tv ) == -1 )
 	    BUG();
 	add_randomness( &tv.tv_sec, sizeof(tv.tv_sec), 1 );
 	add_randomness( &tv.tv_nsec, sizeof(tv.tv_nsec), 1 );
     }
-#elif defined (HAVE_TIMES)
+  #else /* use times */
+    #ifndef HAVE_DOSISH_SYSTEM
     {	struct tms buf;
-        if( times( &buf ) == -1 )
-	    BUG();
+	times( &buf );
 	add_randomness( &buf, sizeof buf, 1 );
     }
-#endif
-#ifdef HAVE_GETRUSAGE
-#ifndef RUSAGE_SELF
-#ifdef __GCC__
-#warning There is no RUSAGE_SELF on this system
-#endif
-#else
+    #endif
+  #endif
+  #ifdef HAVE_GETRUSAGE
+    #ifndef RUSAGE_SELF
+      #ifdef __GCC__
+	#warning There is no RUSAGE_SELF on this system
+      #endif
+    #else
     {	struct rusage buf;
         /* QNX/Neutrino does return ENOSYS - so we just ignore it and
          * add whatever is in buf.  In a chroot environment it might not
-         * work at all (i.e. because /proc/ is not accessible), so we better
+         * work at all (i.e. because /proc/ is not accessible), so we better 
          * ignore all error codes and hope for the best
          */
         getrusage( RUSAGE_SELF, &buf );
-
+        
 	add_randomness( &buf, sizeof buf, 1 );
-	wipememory( &buf, sizeof buf );
+	memset( &buf, 0, sizeof buf );
     }
-#endif
-#endif
+    #endif
+  #endif
     /* time and clock are available on all systems - so
      * we better do it just in case one of the above functions
      * didn't work */
@@ -894,23 +725,25 @@ gather_faked( void (*add)(const void*, size_t, int), int requester,
 		   "it run - it is in no way a strong RNG!\n\n"
 		   "DON'T USE ANY DATA GENERATED BY THIS PROGRAM!!\n\n"));
 	initialized=1;
-#ifdef HAVE_RAND
+      #ifdef HAVE_RAND
 	srand(make_timestamp()*getpid());
-#else
+      #else
 	srandom(make_timestamp()*getpid());
-#endif
+      #endif
     }
 
-    p = buffer = xmalloc( length );
+    p = buffer = m_alloc( length );
     n = length;
-#ifdef HAVE_RAND
+  #ifdef HAVE_RAND
     while( n-- )
 	*p++ = ((unsigned)(1 + (int) (256.0*rand()/(RAND_MAX+1.0)))-1);
-#else
+  #else
     while( n-- )
 	*p++ = ((unsigned)(1 + (int) (256.0*random()/(RAND_MAX+1.0)))-1);
-#endif
+  #endif
     add_randomness( buffer, length, requester );
-    xfree(buffer);
+    m_free(buffer);
     return 0; /* okay */
 }
+
+
