@@ -1,12 +1,11 @@
 /* revoke.c
- * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003,
- *               2004 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
+ * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -15,7 +14,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
 #include <config.h>
@@ -59,15 +59,15 @@ revocation_reason_build_cb( PKT_signature *sig, void *opaque )
 	ud = native_to_utf8( reason->desc );
 	buflen += strlen(ud);
     }
-    buffer = xmalloc( buflen );
+    buffer = m_alloc( buflen );
     *buffer = reason->code;
     if( ud ) {
 	memcpy(buffer+1, ud, strlen(ud) );
-	xfree( ud );
+	m_free( ud );
     }
 
     build_sig_subpkt( sig, SIGSUBPKT_REVOC_REASON, buffer, buflen );
-    xfree( buffer );
+    m_free( buffer );
     return 0;
 }
 
@@ -89,7 +89,7 @@ export_minimal_pk(IOBUF out,KBNODE keyblock,
   node=find_kbnode(keyblock,PKT_PUBLIC_KEY);
   if(!node)
     {
-      log_error("key incomplete\n");
+      log_error(_("key incomplete\n"));
       return G10ERR_GENERAL;
     }
 
@@ -142,7 +142,7 @@ export_minimal_pk(IOBUF out,KBNODE keyblock,
 	    break;
 	  else
 	    {
-	      log_error(_("key %s has no user IDs\n"),keystr(keyid));
+	      log_error(_("key %08lX incomplete\n"),(ulong)keyid[1]);
 	      return G10ERR_GENERAL;
 	    }
 	}
@@ -195,7 +195,7 @@ export_minimal_pk(IOBUF out,KBNODE keyblock,
  * Generate a revocation certificate for UNAME via a designated revoker
  */
 int
-gen_desig_revoke( const char *uname, STRLIST locusr )
+gen_desig_revoke( const char *uname )
 {
     int rc = 0;
     armor_filter_context_t afx;
@@ -209,13 +209,11 @@ gen_desig_revoke( const char *uname, STRLIST locusr )
     KBNODE keyblock=NULL,node;
     u32 keyid[2];
     int i,any=0;
-    SK_LIST sk_list=NULL;
 
-    if( opt.batch )
-      {
-	log_error(_("can't do this in batch mode\n"));
+    if( opt.batch ) {
+	log_error(_("sorry, can't do this in batch mode\n"));
 	return G10ERR_GENERAL;
-      }
+    }
 
     memset( &afx, 0, sizeof afx);
 
@@ -223,7 +221,7 @@ gen_desig_revoke( const char *uname, STRLIST locusr )
     classify_user_id (uname, &desc);
     rc = desc.mode? keydb_search (kdbhd, &desc, 1) : G10ERR_INV_USER_ID;
     if (rc) {
-	log_error (_("key \"%s\" not found: %s\n"),uname, g10_errstr (rc));
+	log_error (_("key `%s' not found: %s\n"),uname, g10_errstr (rc));
 	goto leave;
     }
 
@@ -245,13 +243,6 @@ gen_desig_revoke( const char *uname, STRLIST locusr )
 
     keyid_from_pk(pk,keyid);
 
-    if(locusr)
-      {
-	rc=build_sk_list(locusr,&sk_list,0,PUBKEY_USAGE_CERT);
-	if(rc)
-	  goto leave;
-      }
-
     /* Are we a designated revoker for this key? */
 
     if(!pk->revkey && pk->numrevkeys)
@@ -259,59 +250,51 @@ gen_desig_revoke( const char *uname, STRLIST locusr )
 
     for(i=0;i<pk->numrevkeys;i++)
       {
-	SK_LIST list;
-
 	if(sk)
 	  free_secret_key(sk);
 
-	if(sk_list)
-	  {
-	    for(list=sk_list;list;list=list->next)
-	      {
-		byte fpr[MAX_FINGERPRINT_LEN];
-		size_t fprlen;
+	sk=m_alloc_clear(sizeof(*sk));
 
-		fingerprint_from_sk(list->sk,fpr,&fprlen);
-
-		/* Don't get involved with keys that don't have 160
-		   bit fingerprints */
-		if(fprlen!=20)
-		  continue;
-
-		if(memcmp(fpr,pk->revkey[i].fpr,20)==0)
-		  break;
-	      }
-
-	    if(list)
-	      sk=copy_secret_key(NULL,list->sk);
-	    else
-	      continue;
-	  }
-	else
-	  {
-	    sk=xmalloc_secure_clear(sizeof(*sk));
-	    rc=get_seckey_byfprint(sk,pk->revkey[i].fpr,MAX_FINGERPRINT_LEN);
-	  }
+	rc=get_seckey_byfprint(sk,pk->revkey[i].fpr,MAX_FINGERPRINT_LEN);
 
 	/* We have the revocation key */
 	if(!rc)
 	  {
-	    PKT_signature *revkey = NULL;
+	    size_t n;
+	    char *p;
+	    u32 sk_keyid[2];
+	    PKT_signature *revkey=NULL;
 
-	    any = 1;
+	    any=1;
+	    keyid_from_sk(sk,sk_keyid);
 
-            print_pubkey_info (NULL, pk);
-	    tty_printf ("\n");
+	    tty_printf("\npub  %4u%c/%08lX %s   ",
+		       nbits_from_pk( pk ),
+		       pubkey_letter( pk->pubkey_algo ),
+		       (ulong)keyid[1], datestr_from_pk(pk) );
 
-	    tty_printf (_("To be revoked by:\n"));
-            print_seckey_info (sk);
+	    p = get_user_id( keyid, &n );
+	    tty_print_utf8_string( p, n );
+	    m_free(p);
+	    tty_printf("\n\n");
 
+	    tty_printf(_("To be revoked by:\n"));
+
+	    tty_printf("\nsec  %4u%c/%08lX %s   ",
+		       nbits_from_sk( sk ),
+		       pubkey_letter( sk->pubkey_algo ),
+		       (ulong)sk_keyid[1], datestr_from_sk(sk) );
+
+	    p = get_user_id( sk_keyid, &n );
+	    tty_print_utf8_string( p, n );
+	    m_free(p);
+	    tty_printf("\n");
 	    if(pk->revkey[i].class&0x40)
 	      tty_printf(_("(This is a sensitive revocation key)\n"));
 	    tty_printf("\n");
 
 	    if( !cpr_get_answer_is_yes("gen_desig_revoke.okay",
-         _("Create a designated revocation certificate for this key? (y/N) ")))
+		       _("Create a revocation certificate for this key? ")) )
 	      continue;
 
 	    /* get the reason for the revocation (this is always v4) */
@@ -330,8 +313,7 @@ gen_desig_revoke( const char *uname, STRLIST locusr )
 	      goto leave;
 
 	    afx.what = 1;
-	    afx.hdrlines = "Comment: A designated revocation certificate"
-	      " should follow\n";
+	    afx.hdrlines = "Comment: A revocation certificate should follow\n";
 	    iobuf_push_filter( out, armor_filter, &afx );
 
 	    /* create it */
@@ -408,7 +390,7 @@ gen_desig_revoke( const char *uname, STRLIST locusr )
       }
 
     if(!any)
-      log_error(_("no revocation keys found for \"%s\"\n"),uname);
+      log_error(_("no revocation keys found for `%s'\n"),uname);
 
   leave:
     if( pk )
@@ -418,8 +400,6 @@ gen_desig_revoke( const char *uname, STRLIST locusr )
     if( sig )
 	free_seckey_enc( sig );
 
-    release_sk_list(sk_list);
-
     if( rc )
 	iobuf_cancel(out);
     else
@@ -449,11 +429,10 @@ gen_revoke( const char *uname )
     struct revocation_reason_info *reason = NULL;
     KEYDB_SEARCH_DESC desc;
 
-    if( opt.batch )
-      {
-	log_error(_("can't do this in batch mode\n"));
+    if( opt.batch ) {
+	log_error(_("sorry, can't do this in batch mode\n"));
 	return G10ERR_GENERAL;
-      }
+    }
 
     memset( &afx, 0, sizeof afx);
     init_packet( &pkt );
@@ -464,12 +443,11 @@ gen_revoke( const char *uname )
     kdbhd = keydb_new (1);
     classify_user_id (uname, &desc);
     rc = desc.mode? keydb_search (kdbhd, &desc, 1) : G10ERR_INV_USER_ID;
-    if (rc)
-      {
-	log_error (_("secret key \"%s\" not found: %s\n"),
+    if (rc) {
+	log_error (_("secret key `%s' not found: %s\n"),
                    uname, g10_errstr (rc));
 	goto leave;
-      }
+    }
 
     rc = keydb_get_keyblock (kdbhd, &keyblock );
     if( rc ) {
@@ -486,7 +464,18 @@ gen_revoke( const char *uname )
      * it's used all over the source */
     sk = node->pkt->pkt.secret_key;
     keyid_from_sk( sk, sk_keyid );
-    print_seckey_info (sk);
+    tty_printf("\nsec  %4u%c/%08lX %s   ",
+	      nbits_from_sk( sk ),
+	      pubkey_letter( sk->pubkey_algo ),
+	      (ulong)sk_keyid[1], datestr_from_sk(sk) );
+    {
+	size_t n;
+	char *p = get_user_id( sk_keyid, &n );
+	tty_print_utf8_string( p, n );
+	m_free(p);
+	tty_printf("\n");
+    }
+    pk = m_alloc_clear( sizeof *pk );
 
     /* FIXME: We should get the public key direct from the secret one */
 
@@ -511,11 +500,10 @@ gen_revoke( const char *uname )
 
     tty_printf("\n");
     if( !cpr_get_answer_is_yes("gen_revoke.okay",
-		  _("Create a revocation certificate for this key? (y/N) ")) )
-      {
+			_("Create a revocation certificate for this key? ")) ){
 	rc = 0;
 	goto leave;
-      }
+    }
 
     if(sk->version>=4 || opt.force_v4_certs) {
       /* get the reason for the revocation */
@@ -531,15 +519,11 @@ gen_revoke( const char *uname )
 	log_error(_("unknown protection algorithm\n"));
 	rc = G10ERR_PUBKEY_ALGO;
 	break;
-      case -3:
-	tty_printf (_("Secret parts of primary key are not available.\n"));
-        rc = G10ERR_NO_SECKEY;
-        break;
       case 0:
 	tty_printf(_("NOTE: This key is not protected!\n"));
 	break;
       default:
-        rc = check_secret_key( sk, 0 );
+	rc = check_secret_key( sk, 0 );
 	break;
     }
     if( rc )
@@ -565,11 +549,9 @@ gen_revoke( const char *uname )
 	goto leave;
     }
 
-    if(PGP2 || PGP6 || PGP7 || PGP8)
+    if(opt.pgp2 || opt.pgp6 || opt.pgp7)
       {
-	/* Use a minimal pk for PGPx mode, since PGP can't import bare
-	   revocation certificates. */
-	rc=export_minimal_pk(out,pub_keyblock,sig,NULL);
+	rc=export_minimal_pk(out,NULL /*pub_keyblock*/,sig,NULL);
 	if(rc)
 	  goto leave;
       }
@@ -624,8 +606,7 @@ ask_revocation_reason( int key_rev, int cert_rev, int hint )
     const char *code_text = NULL;
 
     do {
-        code=-1;
-	xfree(description);
+	m_free(description);
 	description = NULL;
 
 	tty_printf(_("Please select the reason for the revocation:\n"));
@@ -652,11 +633,11 @@ ask_revocation_reason( int key_rev, int cert_rev, int hint )
 	      return NULL; /* cancel */
 	    if( hint && !*answer )
 		n = hint;
-	    else if(!digitp( answer ) )
+	    else if(!isdigit( *answer ) )
  	        n = -1;
 	    else
 		n = atoi(answer);
-	    xfree(answer);
+	    m_free(answer);
 	    if( n == 0 ) {
 	        code = 0x00; /* no particular reason */
 		code_text = text_0;
@@ -688,25 +669,25 @@ ask_revocation_reason( int key_rev, int cert_rev, int hint )
 	    trim_trailing_ws( answer, strlen(answer) );
 	    cpr_kill_prompt();
 	    if( !*answer ) {
-		xfree(answer);
+		m_free(answer);
 		break;
 	    }
 
 	    {
 		char *p = make_printable_string( answer, strlen(answer), 0 );
-		xfree(answer);
+		m_free(answer);
 		answer = p;
 	    }
 
 	    if( !description )
-		description = xstrdup(answer);
+		description = m_strdup(answer);
 	    else {
-		char *p = xmalloc( strlen(description) + strlen(answer) + 2 );
+		char *p = m_alloc( strlen(description) + strlen(answer) + 2 );
 		strcpy(stpcpy(stpcpy( p, description),"\n"),answer);
-		xfree(description);
+		m_free(description);
 		description = p;
 	    }
-	    xfree(answer);
+	    m_free(answer);
 	}
 
 	tty_printf(_("Reason for revocation: %s\n"), code_text );
@@ -716,9 +697,9 @@ ask_revocation_reason( int key_rev, int cert_rev, int hint )
 	    tty_printf("%s\n", description );
 
     } while( !cpr_get_answer_is_yes("ask_revocation_reason.okay",
-					    _("Is this okay? (y/N) "))  );
+					    _("Is this okay? "))  );
 
-    reason = xmalloc( sizeof *reason );
+    reason = m_alloc( sizeof *reason );
     reason->code = code;
     reason->desc = description;
     return reason;
@@ -728,7 +709,7 @@ void
 release_revocation_reason_info( struct revocation_reason_info *reason )
 {
     if( reason ) {
-	xfree( reason->desc );
-	xfree( reason );
+	m_free( reason->desc );
+	m_free( reason );
     }
 }
