@@ -1,12 +1,11 @@
 /* util.h
- * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
- *               2006 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  *
  * This file is part of GNUPG.
  *
  * GNUPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
+ * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * GNUPG is distributed in the hope that it will be useful,
@@ -15,25 +14,20 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 #ifndef G10_UTIL_H
 #define G10_UTIL_H
 
-#include <stdarg.h>
+#if defined (__MINGW32__) || defined (__CYGWIN32__)
+# include <stdarg.h>
+#endif
 
 #include "types.h"
 #include "errors.h"
 #include "types.h"
 #include "mpi.h"
-#include "compat.h"
-
-/* GCC attributes.  */
-#if __GNUC__ >= 4
-# define GNUPG_GCC_A_SENTINEL(a) __attribute__ ((sentinel(a)))
-#else
-# define GNUPG_GCC_A_SENTINEL(a)
-#endif
 
 
 typedef struct {
@@ -57,7 +51,6 @@ typedef struct {
 	 const char *last;
 	 void *aliases;
 	 const void *cur_alias;
-         void *iio_list;
      } internal;	    /* DO NOT CHANGE */
 } ARGPARSE_ARGS;
 
@@ -77,7 +70,6 @@ const char *log_get_name(void);
 void log_set_pid( int pid );
 int  log_get_errorcount( int clear );
 void log_inc_errorcount(void);
-int log_set_strict(int val);
 void g10_log_hexdump( const char *text, const char *buf, size_t len );
 
 #if defined (__riscos__) \
@@ -89,12 +81,19 @@ void g10_log_hexdump( const char *text, const char *buf, size_t len );
 			    __attribute__ ((noreturn, format (printf,1,2)));
   void g10_log_error( const char *fmt, ... ) __attribute__ ((format (printf,1,2)));
   void g10_log_info( const char *fmt, ... )  __attribute__ ((format (printf,1,2)));
-  void g10_log_warning( const char *fmt, ... )  __attribute__ ((format (printf,1,2)));
   void g10_log_debug( const char *fmt, ... ) __attribute__ ((format (printf,1,2)));
+  void g10_log_fatal_f( const char *fname, const char *fmt, ... )
+			    __attribute__ ((noreturn, format (printf,2,3)));
+  void g10_log_error_f( const char *fname, const char *fmt, ... )
+			    __attribute__ ((format (printf,2,3)));
+  void g10_log_info_f( const char *fname, const char *fmt, ... )
+			    __attribute__ ((format (printf,2,3)));
+  void g10_log_debug_f( const char *fname,  const char *fmt, ... )
+			    __attribute__ ((format (printf,2,3)));
 #ifndef __riscos__
-#define BUG() g10_log_bug0(  __FILE__ , __LINE__, __FUNCTION__ )
+  #define BUG() g10_log_bug0(  __FILE__ , __LINE__, __FUNCTION__ )
 #else
-#define BUG() g10_log_bug0(  __FILE__ , __LINE__, __func__ )
+  #define BUG() g10_log_bug0(  __FILE__ , __LINE__, __func__ )
 #endif
 #else
   void g10_log_bug( const char *fmt, ... );
@@ -102,9 +101,12 @@ void g10_log_hexdump( const char *text, const char *buf, size_t len );
   void g10_log_fatal( const char *fmt, ... );
   void g10_log_error( const char *fmt, ... );
   void g10_log_info( const char *fmt, ... );
-  void g10_log_warning( const char *fmt, ... );
   void g10_log_debug( const char *fmt, ... );
-#define BUG() g10_log_bug0( __FILE__ , __LINE__ )
+  void g10_log_fatal_f( const char *fname, const char *fmt, ... );
+  void g10_log_error_f( const char *fname, const char *fmt, ... );
+  void g10_log_info_f( const char *fname, const char *fmt, ... );
+  void g10_log_debug_f( const char *fname, const char *fmt, ... );
+  #define BUG() g10_log_bug0( __FILE__ , __LINE__ )
 #endif
 
 #define log_hexdump g10_log_hexdump
@@ -113,8 +115,11 @@ void g10_log_hexdump( const char *text, const char *buf, size_t len );
 #define log_fatal   g10_log_fatal
 #define log_error   g10_log_error
 #define log_info    g10_log_info
-#define log_warning g10_log_warning
 #define log_debug   g10_log_debug
+#define log_fatal_f g10_log_fatal_f
+#define log_error_f g10_log_error_f
+#define log_info_f  g10_log_info_f
+#define log_debug_f g10_log_debug_f
 
 
 /*-- errors.c --*/
@@ -133,36 +138,39 @@ const char *strusage( int level );
 
 
 /*-- dotlock.c --*/
-#include "../include/dotlock.h"
+struct dotlock_handle;
+typedef struct dotlock_handle *DOTLOCK;
+
+void disable_dotlock(void);
+DOTLOCK create_dotlock( const char *file_to_lock );
+int make_dotlock( DOTLOCK h, long timeout );
+int release_dotlock( DOTLOCK h );
+void remove_lockfiles (void);
 
 /*-- fileutil.c --*/
-char * make_basename(const char *filepath, const char *inputpath);
+char * make_basename(const char *filepath);
 char * make_dirname(const char *filepath);
 char *make_filename( const char *first_part, ... );
 int compare_filenames( const char *a, const char *b );
-int same_file_p (const char *name1, const char *name2);
 const char *print_fname_stdin( const char *s );
 const char *print_fname_stdout( const char *s );
-int is_file_compressed(const char *s, int *r_status);
+int  is_file_compressed(const char *s, int *r_status);
+
 
 /*-- miscutil.c --*/
 u32 make_timestamp(void);
 u32 scan_isodatestr( const char *string );
-u32 isotime2seconds (const char *string);
+u32 add_days_to_timestamp( u32 stamp, u16 days );
 const char *strtimevalue( u32 stamp );
 const char *strtimestamp( u32 stamp ); /* GMT */
-const char *isotimestamp( u32 stamp ); /* GMT with hh:mm:ss */
 const char *asctimestamp( u32 stamp ); /* localized */
 void print_string( FILE *fp, const byte *p, size_t n, int delim );
-void print_string2( FILE *fp, const byte *p, size_t n, int delim, int delim2 );
 void  print_utf8_string( FILE *fp, const byte *p, size_t n );
 void  print_utf8_string2( FILE *fp, const byte *p, size_t n, int delim);
 char *make_printable_string( const byte *p, size_t n, int delim );
 int answer_is_yes_no_default( const char *s, int def_answer );
 int answer_is_yes( const char *s );
 int answer_is_yes_no_quit( const char *s );
-int answer_is_okay_cancel (const char *s, int def_answer);
-int match_multistr(const char *multistr,const char *match);
 
 /*-- strgutil.c --*/
 void free_strlist( STRLIST sl );
@@ -185,18 +193,19 @@ unsigned int check_trailing_chars( const byte *line, unsigned int len,
                                    const char *trimchars );
 unsigned int check_trailing_ws( const byte *line, unsigned int len );
 int string_count_chr( const char *string, int c );
-int has_invalid_email_chars (const char *s);
-int is_valid_mailbox (const char *name);
 int set_native_charset( const char *newset );
 const char* get_native_charset(void);
 char *native_to_utf8( const char *string );
 char *utf8_to_native( const char *string, size_t length, int delim);
-char *string_to_utf8 (const char *string);
+int  check_utf8_string( const char *string );
 
 int ascii_isupper (int c);
 int ascii_islower (int c);
+int ascii_toupper (int c);
+int ascii_tolower (int c);
+int ascii_strcasecmp( const char *a, const char *b );
+int ascii_strncasecmp( const char *a, const char *b, size_t n);
 int ascii_memcasecmp( const char *a, const char *b, size_t n);
-char *ascii_strlwr (char *s);
 
 #ifndef HAVE_STPCPY
 char *stpcpy(char *a,const char *b);
@@ -204,6 +213,9 @@ char *stpcpy(char *a,const char *b);
 #ifndef HAVE_STRLWR
 char *strlwr(char *a);
 #endif
+#ifndef HAVE_STRSEP
+char *strsep (char **stringp, const char *delim);
+#endif
 #ifndef HAVE_STRCASECMP
 int strcasecmp( const char *, const char *b);
 #endif
@@ -211,128 +223,70 @@ int strcasecmp( const char *, const char *b);
 int strncasecmp (const char *, const char *b, size_t n);
 #endif
 #ifndef HAVE_STRTOUL
-#define strtoul(a,b,c)  ((unsigned long)strtol((a),(b),(c)))
+  #define strtoul(a,b,c)  ((unsigned long)strtol((a),(b),(c)))
 #endif
 #ifndef HAVE_MEMMOVE
-#define memmove(d, s, n) bcopy((s), (d), (n))
+  #define memmove(d, s, n) bcopy((s), (d), (n))
 #endif
 
-/*-- membuf.c --*/
-/* The definition of the structure is private, we only need it here,
-   so it can be allocated on the stack. */
-struct private_membuf_s {
-  size_t len;
-  size_t size;
-  char *buf;
-  int out_of_core;
-};
-
-typedef struct private_membuf_s membuf_t;
-
-void init_membuf (membuf_t *mb, int initiallen);
-void put_membuf  (membuf_t *mb, const void *buf, size_t len);
-void put_membuf_str (membuf_t *mb, const char *buf);
-void *get_membuf (membuf_t *mb, size_t *len);
-
-
-
-#if defined (_WIN32)
+#if defined (__MINGW32__) || defined (__CYGWIN32__)
 /*-- w32reg.c --*/
 char *read_w32_registry_string( const char *root,
 				const char *dir, const char *name );
 int write_w32_registry_string(const char *root, const char *dir,
                               const char *name, const char *value);
 
-#endif /*_WIN32*/
-
 /*-- strgutil.c --*/
-char *xasprintf (const char *fmt, ...);
-char *xtryasprintf (const char *fmt, ...);
-char *xtryvasprintf (const char *fmt, va_list arg_ptr);
-char *strconcat (const char *s1, ...) GNUPG_GCC_A_SENTINEL(0);
-
-/*-- pka.c --*/
-char *get_pka_info (const char *address, void *fprbuf, size_t fprbuflen);
-
-/*-- cert.c --*/
-int get_cert (const char *name, int want_ipgp, size_t max_size, IOBUF *iobuf,
-              unsigned char **fpr,size_t *fpr_len,char **url);
-
-/*-- convert.c --*/
-int hex2bin (const char *string, void *buffer, size_t length);
-int hexcolon2bin (const char *string, void *buffer, size_t length);
-char *bin2hex (const void *buffer, size_t length, char *stringbuf);
-char *bin2hexcolon (const void *buffer, size_t length, char *stringbuf);
-const char *hex2str (const char *hexstring,
-                     char *buffer, size_t bufsize, size_t *buflen);
-char *hex2str_alloc (const char *hexstring, size_t *r_count);
-
+int vasprintf ( char **result, const char *format, va_list args);
+#endif
 
 /**** other missing stuff ****/
 #ifndef HAVE_ATEXIT  /* For SunOS */
-#define atexit(a)    (on_exit((a),0))
+  #define atexit(a)    (on_exit((a),0))
 #endif
 
 #ifndef HAVE_RAISE
-#define raise(a) kill(getpid(), (a))
+  #define raise(a) kill(getpid(), (a))
 #endif
 
-/*-- Replacement functions from funcname.c --*/
-
-
-
 /******** some macros ************/
 #ifndef STR
-#define STR(v) #v
+  #define STR(v) #v
 #endif
 #define STR2(v) STR(v)
 #define DIM(v) (sizeof(v)/sizeof((v)[0]))
 #define DIMof(type,member)   DIM(((type *)0)->member)
 
-#define wipememory2(_ptr,_set,_len) do { volatile char *_vptr=(volatile char *)(_ptr); size_t _vlen=(_len); while(_vlen) { *_vptr=(_set); _vptr++; _vlen--; } } while(0)
-#define wipememory(_ptr,_len) wipememory2(_ptr,0,_len)
-
-/*-- macros to replace ctype ones and avoid locale problems --*/
-#define spacep(p)   (*(p) == ' ' || *(p) == '\t')
-#define digitp(p)   (*(p) >= '0' && *(p) <= '9')
-#define hexdigitp(a) (digitp (a)                     \
-                      || (*(a) >= 'A' && *(a) <= 'F')  \
-                      || (*(a) >= 'a' && *(a) <= 'f'))
-/* the atoi macros assume that the buffer has only valid digits */
-#define atoi_1(p)   (*(p) - '0' )
-#define atoi_2(p)   ((atoi_1(p) * 10) + atoi_1((p)+1))
-#define atoi_4(p)   ((atoi_2(p) * 100) + atoi_2((p)+2))
-#define xtoi_1(p)   (*(p) <= '9'? (*(p)- '0'): \
-                     *(p) <= 'F'? (*(p)-'A'+10):(*(p)-'a'+10))
-#define xtoi_2(p)   ((xtoi_1(p) * 16) + xtoi_1((p)+1))
-
 /******* RISC OS stuff ***********/
 #ifdef __riscos__
-int riscos_load_module(const char *name, const char * const path[], int fatal);
-int riscos_get_filetype_from_string(const char *string, int len);
-int riscos_get_filetype(const char *filename);
-void riscos_set_filetype_by_number(const char *filename, int type);
-void riscos_set_filetype_by_mimetype(const char *filename, const char *mimetype);
+/* needed for strcasecmp() */
+#include <strings.h>
+/* needed for filename munging */
+#include <unixlib/local.h>
+/* needed for image file system feature */
+#include <unixlib/features.h>
+void riscos_global_defaults(void);
+#define RISCOS_GLOBAL_STATICS(a) const char *__dynamic_da_name = (a);
+void riscos_set_filetype(const char *filename, const char *mimetype);
 pid_t riscos_getpid(void);
 int riscos_kill(pid_t pid, int sig);
 int riscos_access(const char *path, int amode);
 int riscos_getchar(void);
-char *riscos_make_basename(const char *filepath, const char *inputpath);
-int riscos_check_regexp(const char *exp, const char *string, int debug);
-int riscos_fdopenfile(const char *filename, const int allow_write);
-void riscos_close_fds(void);
-int riscos_renamefile(const char *old, const char *new);
-char *riscos_gstrans(const char *old);
-void riscos_not_implemented(const char *feature);
+int fdopenfile(const char *filename, const int allow_write);
+void close_fds(void);
+int renamefile(const char *old, const char *new);
+char *gstrans(const char *old);
+void not_implemented(const char *feature);
 #ifdef DEBUG
-void riscos_dump_fdlist(void);
-void riscos_list_openfiles(void);
+void dump_fdlist(void);
+void list_openfiles(void);
 #endif
 #ifndef __RISCOS__C__
-#define getpid riscos_getpid
-#define kill(a,b) riscos_kill((a),(b))
-#define access(a,b) riscos_access((a),(b))
+  #define getpid riscos_getpid
+  #define kill(a,b) riscos_kill((a),(b))
+  #define access(a,b) riscos_access((a),(b))
 #endif /* !__RISCOS__C__ */
 #endif /* __riscos__ */
 
 #endif /*G10_UTIL_H*/
+
